import javax.swing.JTextPane;



import graph.*;

public class Dijkstra extends Thread{
	private Graph bspGraph;
	private String startKnoten;
	private String zielKnoten;
	private JTextPane textfeld;

	public Dijkstra(Graph pGraph, String pStart, String pZiel,
			 JTextPane pText) {
		bspGraph = pGraph;
		startKnoten = pStart;
		zielKnoten = pZiel;
		textfeld = pText;
	}

	public void zuruecksetzenDijkstraVertices(Graph pGraph) {
		List<Vertex> knoten = pGraph.getVertices();
		while (knoten.hasAccess()) {
			DijkstraVertex djk = (DijkstraVertex) knoten.getContent();
			djk.setzeDistanz(-1);
			((DijkstraVertex) knoten.getContent()).setzeVorgaenger(null);
			((DijkstraVertex) knoten.getContent()).setMark(false);
			knoten.next();
		}
	}

	public void berechneDijkstra(Graph pGraph, DijkstraVertex startKnoten) {
		// Graph vorbereiten und Liste der zu bearbeitenden Knoten
		// (aktuelleKnoten) vorbereiten.
		this.zuruecksetzenDijkstraVertices(pGraph);
		List<DijkstraVertex> aktuelleKnoten = new List<DijkstraVertex>();

		// Startknoten mit Distanz 0 in die Liste der zu bearbeitenden Knoten
		// (aktuelleKnoten)
		// einfuegen.
		aktuelleKnoten.append(startKnoten);
		startKnoten.setzeDistanz(0);

		// Distanzen aller Knoten vom Startknoten aus mit Dijkstra berechnen.
		while (!aktuelleKnoten.isEmpty() ) {
			// Knoten mit geringstem Distanzeintrag (aktuell) aus der Liste der
			// aktuell zu
			// bearbeitenden Knoten (aktuelleKnoten) ermitteln.
			aktuelleKnoten.toFirst();
			DijkstraVertex aktuell = (DijkstraVertex) aktuelleKnoten
					.getContent();
			while (aktuelleKnoten.hasAccess()) {
				if (((DijkstraVertex) aktuelleKnoten.getContent()).gibDistanz() < aktuell
						.gibDistanz()) {
					aktuell = (DijkstraVertex) aktuelleKnoten.getContent();
				}
				aktuelleKnoten.next();
			}

			// Aktuellen Knoten (aktuell) expandieren, seine Nachbarn in die
			// Liste der
			// zu bearbeitenden Knoten (aktuelleKnoten) einfuegen und die
			// Distanzen der Nachbarn
			// ggf. aktualisieren.
			List<Vertex> nachbarn = pGraph.getNeighbours(aktuell);
			nachbarn.toFirst();
			while (nachbarn.hasAccess()) {
				// Nachbarn bearbeiten, sofern er nicht schon markiert, d.h.
				// fertig abgearbeitet, ist.
				DijkstraVertex nachbar = (DijkstraVertex) nachbarn.getContent();
				if (!nachbar.isMarked()) {
					aktuelleKnoten.append(nachbar); // Knoten der Liste der zu
													// bearbeitenden Knoten
													// hinzufuegen.

					// Kante (min) vom Knoten aktuell zum Knoten nachbar suchen.
					Edge kante = pGraph.getEdge(aktuell, nachbar);

					// Aktualisierung durchfuehren, wenn es erforderlich ist.
					if (nachbar.gibDistanz() == -1
							|| aktuell.gibDistanz() + kante.getWeight() < nachbar
									.gibDistanz()) {
						nachbar.setzeDistanz(aktuell.gibDistanz()
								+ kante.getWeight());
						nachbar.setzeVorgaenger(aktuell);
					}

				}

				nachbarn.next();
			}

			// Knoten aktuell aus der Liste der zu bearbeitenden Knoten
			// entfernen
			// und als abgearbeitet markieren.
			aktuelleKnoten.toFirst();
			while (aktuelleKnoten.hasAccess()) {
				if (aktuelleKnoten.getContent() == aktuell) {
					aktuelleKnoten.remove();
				}
				aktuelleKnoten.next();
			}
			aktuell.setMark(true);

		}

	}
	
 @Override
 public void run() {
	 DijkstraVertex sk =(DijkstraVertex) bspGraph.getVertex(startKnoten);
	 this.berechneDijkstra(bspGraph,sk);
	 DijkstraVertex zk = (DijkstraVertex)bspGraph.getVertex(zielKnoten);
	 DijkstraVertex alt = zk;
	 while (alt!=sk){
		 DijkstraVertex vorgaenger = alt.gibVorgaenger();
		 Edge kante = bspGraph.getEdge(vorgaenger, alt);
		 kante.setMark(true);
		 alt = vorgaenger;
	 }
	 bspGraph.setAllVertexMarks(false);
	 textfeld.setText("Weglaenge von "+startKnoten + " nach "+zielKnoten+": "+zk.gibDistanz());
 }
	

}
