import graph.*;

import javax.swing.JTextPane;

public class Prim {
	private Graph bspGraph;
	boolean schritt = false;
	private JTextPane textfeld;
	private String markierteKanten="";

	public Prim(Graph graph,JTextPane pText) {
		bspGraph = graph;
		textfeld = pText;
	}

	 public Graph gibSpannbaumPrim(Graph pGraph){
		    //Knotenmarkierungen des Graphen zuruecksetzen.
		    pGraph.setAllVertexMarks(false);

		    //Ergebnisgraphen (Spannbaum) erstellen und alle Knoten uebertragen.
		    Graph ergebnis = new Graph();
		    List<Vertex> knoten = pGraph.getVertices();
		    knoten.toFirst();
		    while (knoten.hasAccess()){
		      ergebnis.addVertex(knoten.getContent());
		      knoten.next();
		    }

		    //Einen Knoten als Startknoten waehlen und markieren. (Koennte auch im Parameter uebergeben werden.)
		    knoten.toFirst();
		    Vertex startKnoten = knoten.getContent();
		    startKnoten.setMark(true);  

		    //Kantenliste erstellen, die alle Kanten enthaelt, die mit der 
		    //bereits erstellten Zusammenhangskomponente inzident sind. Am Anfang werden
		    //die mit dem Startknoten inzidenten Kanten eingefuegt.
		    List<Edge> kanten = pGraph.getEdges(startKnoten); 

		    //Kanten hinzufuegen, solange noch nicht alle Knoten im Graphen markiert sind.
		    while (!ergebnis.allVerticesMarked()){
		      //Minimale Kante der Kantenliste ermitteln und solche Kanten entfernen, die innerhalb der erschlossenen 
		      //Zusammenhangskomponente liegen.
		      Edge minKante = null;
		      kanten.toFirst();      
		      while (kanten.hasAccess()){
		        //Kante entfernen, wenn sie innerhalb der bereits erschlossenen Zusammenhangskomponente verlaeuft.
		        if (kanten.getContent().getVertices()[0].isMarked() && kanten.getContent().getVertices()[1].isMarked()){
		          kanten.remove();
		        } else {
		          //Auf kleinere Kante ueberpruefen.
		          if (minKante == null || kanten.getContent().getWeight() < minKante.getWeight()){
		            minKante = kanten.getContent();
		          }   
		          kanten.next();
		        }
		      }

		      //Minimale Kante dem Graphen hinzufuegen.
		      ergebnis.addEdge(minKante);
		      minKante.setMark(true);
		      markierteKanten = markierteKanten+minKante.getVertices()[0].getID()+minKante.getVertices()[1].getID()+" ";
		      //Den Knoten ermitteln der durch die minimale Kante der Zusammenhangskomponete hinzugefuegt wird.
		      Vertex neuerKnoten = null;
		      if (minKante.getVertices()[0].isMarked()){
		        neuerKnoten = minKante.getVertices()[1];
		      } else {
		        neuerKnoten = minKante.getVertices()[0];
		      }

		      //Neu verbundenen Knoten markieren.
		      neuerKnoten.setMark(true);

		      //Mit dem neuen Knoten inzidente Kanten in die Kantenliste hinzufuegen.
		      List<Edge> kantenKandidaten = pGraph.getEdges(neuerKnoten);
		      kantenKandidaten.toFirst();
		      while (kantenKandidaten.hasAccess()){
		        kanten.append(kantenKandidaten.getContent());
		        kantenKandidaten.next();
		      }

		    }
		    return ergebnis;
		  }

	public double berechneBaumlaenge(List<Edge> kantenListe) {
		bspGraph.setAllVertexMarks(false);
		double sum = 0;
		kantenListe.toFirst();
		while (kantenListe.hasAccess()) {
			Edge k = kantenListe.getContent();
			//k.setMark(true);
			sum = sum + k.getWeight();
			kantenListe.next();
		}
		return sum;
	}
    
	public void run() {
		synchronized(this) {
		Graph spbPrim = gibSpannbaumPrim(bspGraph);
		double baumlaenge = this.berechneBaumlaenge(spbPrim.getEdges());
		bspGraph.setAllVertexMarks(false);
		textfeld.setText("Spannbaum Prim-Algorithmus\n");
		textfeld.setText(textfeld.getText()+"Reihenfolge gewählter Kanten\n"+markierteKanten+"\nGesamtlänge: "+baumlaenge);
		  
	   }
   }
	
				
 
	
}
